package com.perforce.hive.common;

import com.perforce.hive.connection.ConnectionConfig;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.impl.generic.client.ClientView;
import com.perforce.p4java.impl.generic.client.ClientView.ClientViewMapping;
import com.perforce.p4java.impl.mapbased.client.Client;
import com.perforce.p4java.server.IOptionsServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;

public class ClientHelper {

	private static Logger logger = LoggerFactory.getLogger(ClientHelper.class);

	public static IClient getClient(IOptionsServer p4) throws P4JavaException {
		String user = p4.getUserName();
		String clientName = ConnectionConfig.get().getClientPrefix() + user;
		IClient iclient = p4.getClient(clientName);

		if(iclient == null) {
			iclient = newClient(p4);
		}
		return iclient;
	}
	
	public static IClient newClient(IOptionsServer p4) throws P4JavaException {
		String user = p4.getUserName();
		String clientName = ConnectionConfig.get().getClientPrefix() + user;
		IClient iclient = p4.getClient(clientName);

		// clean up and delete
		if (iclient != null) {
			ClientFactory.clean(p4, iclient);
			p4.deleteClient(iclient.getName(), false);
		}

		// create new workspace
		Client implClient = new Client(p4);
		implClient.setName(clientName);
		p4.createClient(implClient);
		iclient = p4.getClient(clientName);
		iclient.setOwnerName(user);

		File root = new File("upload", clientName);
		iclient.setRoot(root.getAbsolutePath());
		iclient.update();

		return iclient;
	}

	public static void mapClientPath(IClient iclient, String unixPath) throws P4JavaException {
		// create LHS 'depot' mapping
		unixPath = unixPath.replace("\\", "/");
		String lhs = "//" + unixPath + "/...";

		// create RHS 'client' mapping
		String rhs = "//" + iclient.getName() + "/...";

		// add view mapping
		addMapping(iclient, lhs, rhs);
	}
	
	public static void mapClientFile(IClient iclient, String depotFile) throws P4JavaException {
		// create LHS 'depot' mapping
		String lhs = depotFile;

		// create RHS 'client' mapping
		String rhs = "//" + iclient.getName() + "/clientFile";

		// add view mapping
		addMapping(iclient, lhs, rhs);
	}
	
	private static void addMapping(IClient iclient, String lhs, String rhs) throws P4JavaException {
		ClientView clientView = new ClientView();
		clientView.setClient(iclient);
		String line = lhs + " " + rhs;
		ClientViewMapping entry = new ClientViewMapping(0, line);
		clientView.addEntry(entry);
		iclient.setClientView(clientView);
		iclient.update();

		logger.debug("workspace: " + iclient.getName());
		logger.debug("... mapping: " + lhs);
	}
}
