package com.perforce.hive.plugin.docker;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.artifact.DefaultArtifact;
import com.perforce.hive.depot.DepotFactory;
import com.perforce.hive.depot.DepotModel;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

import spark.Request;
import spark.Response;

public class DockerArtifact extends DefaultArtifact implements ArtifactInterface {

	private static Logger logger = LoggerFactory.getLogger(DockerArtifact.class);

	private String dockerID;
	private String version;
	private List<FileItem> fileItems = new ArrayList<>();

	public DockerArtifact(Request request, Response response) throws Exception {
		super(request, response);
	}

	public DockerArtifact(IOptionsServer p4, String repoID) throws Exception {
		super(p4);
		this.repoID = repoID;
	}

	@Override
	protected void processForm(Request request) throws FileUploadException {
		DiskFileItemFactory factory = new DiskFileItemFactory();
		ServletFileUpload up = new ServletFileUpload(factory);
		List<FileItem> items = up.parseRequest(request.raw());

		for (FileItem item : items) {
			if (item.isFormField()) {
				String fieldname = item.getFieldName();
				String fieldvalue = item.getString();

				switch (fieldname) {
				case "repoID":
					repoID = fieldvalue;
					break;
				case "dockerID":
					dockerID = fieldvalue;
					break;
				case "version":
					version = fieldvalue;
					break;
				}
			} else {
				fileItems.add(item);
			}
		}

		// set path
		path = Paths.get(repoID, "docker");
	}

	@Override
	public DepotModel getDepot() throws P4JavaException {
		DepotModel depot = DepotFactory.getByName(p4, repoID);
		return depot;
	}

	@Override
	public void index() throws Exception {
		// no indexing
	}

	@Override
	public void buildIndex() throws Exception {
		// no indexing
	}
}
