package com.perforce.hive.plugin.maven;

import java.nio.file.Paths;
import java.util.List;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.artifact.DefaultArtifact;
import com.perforce.hive.depot.DepotFactory;
import com.perforce.hive.depot.DepotModel;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

import spark.Request;
import spark.Response;

public class MavenArtifact extends DefaultArtifact implements ArtifactInterface {

	private static Logger logger = LoggerFactory.getLogger(MavenArtifact.class);

	public MavenArtifact(Request request, Response response) throws Exception {
		super(request, response);
	}

	public MavenArtifact(IOptionsServer p4, String repoID) throws Exception {
		super(p4);
		this.repoID = repoID;
	}

	@Override
	public void processForm(Request request) throws FileUploadException {
		DiskFileItemFactory factory = new DiskFileItemFactory();
		ServletFileUpload up = new ServletFileUpload(factory);
		List<FileItem> items = up.parseRequest(request.raw());

		String groupID = "";
		String assetID = "";
		String version = "";

		for (FileItem item : items) {
			if (item.isFormField()) {
				String fieldname = item.getFieldName();
				String fieldvalue = item.getString();

				switch (fieldname) {
				case "repoID":
					repoID = fieldvalue;
					break;
				case "groupID":
					groupID = fieldvalue;
					break;
				case "assetID":
					assetID = fieldvalue;
					break;
				case "version":
					version = fieldvalue;
					break;
				}
			} else {
				fileItems.add(item);
			}
		}

		// set path
		String groupPath = groupID.replace(".", "/");
		String id[] = { groupPath, assetID, version };
		path = Paths.get(repoID, id);
	}

	@Override
	public DepotModel getDepot() throws P4JavaException {
		DepotModel depot = DepotFactory.getByName(p4, repoID);
		return depot;
	}

	@Override
	public void index() throws Exception {
		// no indexing
	}

	@Override
	public void buildIndex() throws Exception {
		// no indexing
	}
}
