package com.perforce.hive.plugin.vagrant;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import com.perforce.hive.ErrorModel;
import com.perforce.hive.PluginArtifactFactory;
import com.perforce.hive.api.DeployInterface;
import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.api.artifact.ArtifactType;
import com.perforce.hive.connection.ConnectionSession;
import com.perforce.hive.depot.DepotFactory;
import com.perforce.hive.navigation.NavBar;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

import freemarker.cache.ClassTemplateLoader;
import freemarker.cache.TemplateLoader;
import freemarker.template.Configuration;
import ro.fortsoft.pf4j.Extension;
import ro.fortsoft.pf4j.PluginWrapper;
import spark.ModelAndView;
import spark.Request;
import spark.Response;
import spark.Spark;
import spark.template.freemarker.FreeMarkerEngine;

public class VagrantEndPoint extends VagrantPlugin {

	public VagrantEndPoint(PluginWrapper wrapper) {
		super(wrapper);
	}

	@Extension
	public static class VagrantDeploy implements DeployInterface {

		@Override
		public TemplateLoader getTemplateLoader() throws IOException {
			return new ClassTemplateLoader(VagrantEndPoint.class, "");
		}

		@Override
		public void addDeployEndPoints(String root, Configuration configuration) {

			String path = root + "/" + getType().getName();

			Spark.get(path, (request, response) -> {
				return VagrantEndPoint.getDeploy(request, response);
			}, new FreeMarkerEngine(configuration));

			Spark.post(path, (request, response) -> {
				return VagrantEndPoint.postDeploy(request, response);
			});
		}

		@Override
		public void addAPIEndPoints(String root, Configuration configuration) {
			// Implement custom API end points
		}
	}

	public static ModelAndView getDeploy(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("deploy", request);

		// get type from request parameters
		ArtifactType type = getType();
		model.put("type", type);

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			model.put("depots", DepotFactory.listDeployable(p4, type));
			p4.disconnect();
		} catch (P4JavaException e) {
			return ErrorModel.error(e);
		}

		return new ModelAndView(model, "/deploy/page-upload.html");
	}

	public static String postDeploy(Request request, Response response) throws Exception {
		// get type from request parameters
		ArtifactType type = getType();

		ArtifactInterface artifact = PluginArtifactFactory.getArtifact(type, request, response);
		List<String> upload = artifact.populateMultiPartPost();
		int pending = artifact.updatePending(upload);
		artifact.disconnect();

		// pending change
		return "/submit/" + type.getName() + "?pending=" + pending;
	}
}
